@@ -1,3 +1,18 @@
+#
+# Copyright [1993-2018] [Charles Kooperberg]
+#
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
+#
 unstrip <- function(x)
 {
    dd <- dim(x)
@@ -118,7 +133,7 @@ doldlogspline <- function(q, fit)
     y
 }
 
-plot.oldlogspline <- function(x, n = 100, what = "d", xlim, xlab = "", ylab = "", type = "l", ...)
+plot.oldlogspline <- function(x, n = 100, what = "d", xlim, xlab = "", ylab = "", type = "l", add = FALSE, ...)
 {
     fit <- x
     if(class(fit)!="oldlogspline")
@@ -148,7 +163,8 @@ plot.oldlogspline <- function(x, n = 100, what = "d", xlim, xlab = "", ylab = ""
                 ylab <- ""
         if(missing(type))
                 type <- "l"
-        plot(xx, yy, xlab = xlab, ylab = ylab, type = type, ...)
+        if(add==FALSE)plot(xx, yy, xlab = xlab, ylab = ylab, type = type, ...)
+        else lines(xx,yy, type = type, ...)
 }
 print.oldlogspline <- function(x,...)
 {
@@ -337,6 +353,7 @@ oldlogspline <- function(uncensored, right, left, interval, lbound, ubound,
         SorC <- vector(mode = "integer", length = 35)
         SorC[1] <- 1    # the actual function call
         nsample[6] <- nsample[6]-1
+        if(length(table(sample))<3)stop("Not enough unique values")
         z <- .C("logcensor",
                 as.integer(delete),
                 as.integer(iautoknot),
@@ -422,6 +439,7 @@ logspline <- function(x, lbound, ubound, maxknots=0, knots, nknots=0,
    call <- match.call()
    if(!missing(x))x <- unstrip(x)
    data <- x
+   if(length(table(data))<3)stop("Not enough unique values")
    ilx <- 0; iux <- 0
    if(!missing(lbound)){ilx <- 1;jlx <- lbound}
    if(!missing(ubound)){iux <- 1;jux <- ubound}
@@ -695,7 +713,3 @@ summary.logspline <- function(object,...)
          ", the default (BIC) ", "would have been", round(log(um), 2), "\n"))
    invisible()
 }
-.First.lib <- function(lib, pkg)
-{
-   library.dynam("logspline", pkg, lib)
-}
